// Created by Windward Studios - no copyright is claimed. This code can be used in
// any manner by anyone for any reason. There is no copyright of any kind on it. You may
// use it in commercial products. You may change it without sharing those changes.
// We ask that you keep the "created by Windward Studios" in a comment at the top.

using System;
using System.Data;

namespace net.windward.utils.ado.ODBC
{
	/// <summary>
	/// Wrap an IDataReader returned on a ODBC Execute. All this does is add the GetTimestamp method.
	/// </summary>
	public class WrOdbcDataReader : WrDataReader
	{

		/// <summary>
		/// Create a WrDataReader object wrapping an IDataReader.
		/// </summary>
		/// <param name="data">The IDataReader we are wrapping.</param>
		public WrOdbcDataReader(IDataReader data)
			: base(data)
		{
		}

		/// <summary>
		/// Read a timestamp field and return it as an Int64.
		/// </summary>
		/// <param name="ordinal">The ordinal of the timestamp column.</param>
		/// <returns>The timestamp with [0] as msw and [1] as lsw.</returns>
		public override long GetTimestamp(int ordinal)
		{
			if (reader.GetFieldType(ordinal) == typeof(DateTime))
			{
				DateTime dt = reader.GetDateTime(ordinal);
				return dt.Ticks;
			}
			byte[] data = new byte[8];
			reader.GetBytes(ordinal, 0, data, 0, 8);

			Int64 rtn = 0;
			for (int ind = 0; ind < 8; ind++)
			{
				rtn <<= 8;
				rtn |= data[ind];
			}

			return rtn;
		}
	}
}
