// Created by Windward Studios - no copyright is claimed. This code can be used in
// any manner by anyone for any reason. There is no copyright of any kind on it. You may
// use it in commercial products. You may change it without sharing those changes.
// We ask that you keep the "created by Windward Studios" in a comment at the top.

using System.Data.Common;

namespace net.windward.utils.ado.Oracle
{
	/// <summary>
	/// An instance of an Oracle database running on a computer.
	/// </summary>
	public class WrOracleServer : IWrServer
	{
		private readonly DbProviderFactory provider;
		private readonly string name;

		/// <summary>
		/// Create a Oracle server object.
		/// </summary>
		/// <param name="provider">The provider for this server.</param>
		/// <param name="name">The name of the server.</param>
		public WrOracleServer(DbProviderFactory provider, string name)
		{
			this.provider = provider;
			this.name = name;
		}

		#region IWrServer Members

		/// <summary>
		/// The name of the server.
		/// </summary>
		public string Name
		{
			get { return name; }
		}

		/// <summary>
		/// All databases on this server. As Oracle does not have the concept of databases this will always return 1 database.
		/// </summary>
		/// <param name="credentials">The user credentials to access the database.</param>
		/// <returns>All databases on this server.</returns>
		public IWrDatabase[] GetDatabases(WrCredentials credentials)
		{
			WrOracleDatabase[] rtn = new WrOracleDatabase[1];
			rtn[0] = new WrOracleDatabase(provider, name);
			return rtn;
		}


		/// <summary>
		/// All databases on this server. If a provider does not have the concept of databases (Oracle)
		/// it will always return 1 database.
		/// </summary>
		/// <param name="connStr">The connection string to access the database.</param>
		/// <returns>All databases on this server.</returns>
		public IWrDatabase[] GetDatabases(string connStr)
		{
			WrOracleDatabase[] rtn = new WrOracleDatabase[1];
			rtn[0] = new WrOracleDatabase(provider, name);
			return rtn;
		}

		#endregion

		/// <summary>
		/// Create a database object.
		/// </summary>
		/// <param name="database">The name of the database.</param>
		public IWrDatabase CreateDatabase(string database)
		{
			return new WrOracleDatabase(provider, name);
		}

		/// <summary>
		/// Compares the current instance with another object of the same type.
		/// </summary>
		/// <param name="obj">An object to compare with this instance. </param>
		/// <returns>a 32-bit signed integer that indicates the relative order of the objects being compared.</returns>
		public int CompareTo(object obj)
		{
			WrOracleServer db = obj as WrOracleServer;
			if (db == null)
				return 1;
			return name.CompareTo(db.name);
		}
	}
}
