// Created by Windward Studios - no copyright is claimed. This code can be used in
// any manner by anyone for any reason. There is no copyright of any kind on it. You may
// use it in commercial products. You may change it without sharing those changes.
// We ask that you keep the "created by Windward Studios" in a comment at the top.

using System;
using System.Data;
using System.Data.Common;
using Kailua.net.windward.utils;

namespace net.windward.utils.ado.Unknown
{

	/// <summary>
	/// DbCommand wrapper for an unknown database. We do what we can with this but Set and ExecuteInsert are both unsupported.
	/// </summary>
	class WrUnknownCommand : WrCommand
	{

		/// <summary>
		/// Wraps a DbCommand object and provides a client independent method of setting parameters. For the unknown case
		/// we don't know much but implement what we can.
		/// </summary>
		public WrUnknownCommand(DbProviderFactory provider)
		{
			cmd = provider.CreateCommand();
			cmd.CommandType = CommandType.Text;
		}

		/// <summary>
		/// Set the value of a parameter.
		/// </summary>
		/// <param name="ind">The index of this paramater in the command.</param>
		/// <param name="value">The value of this parameter.</param>
		public override void Set(int ind, object value)
		{
			throw new NotImplementedException("Set not supported for unknown vendors");
		}

		///<summary>
		///Used for inserting a row with an auto primary key. Executes an SQL statement against the Connection object of a .NET Framework 
		/// data provider, and returns the value of the primary key.
		///</summary>
		/// <param name="pkName">The name of the PK column</param>
		///<returns>
		///The value of the primary key for the inserted row.
		///</returns>
		///<exception cref="T:System.InvalidOperationException">The connection does not exist.-or- The connection is not open. </exception><filterpriority>2</filterpriority>
		public override long ExecuteInsert(string pkName)
		{
			throw new NotImplementedException("ExecuteInsert not supported for unknown vendors");
		}

		///<summary>
		///Executes the <see cref="P:System.Data.IDbCommand.CommandText"></see> against the 
		/// <see cref="P:System.Data.IDbCommand.Connection"></see> and builds an <see cref="T:System.Data.IDataReader"></see>.
		///</summary>
		///
		///<returns>
		///An <see cref="T:System.Data.IDataReader"></see> object.
		///</returns>
		///<filterpriority>2</filterpriority>
		public override IDataReader ExecuteReader()
		{
			Trap.trap();
			return new WrUnknownDataReader(cmd.ExecuteReader());
		}

		///<summary>
		///Executes the <see cref="P:System.Data.IDbCommand.CommandText"></see> against the 
		/// <see cref="P:System.Data.IDbCommand.Connection"></see>, and builds an <see cref="T:System.Data.IDataReader"></see> 
		/// using one of the <see cref="T:System.Data.CommandBehavior"></see> values.
		///</summary>
		///
		///<returns>
		///An <see cref="T:System.Data.IDataReader"></see> object.
		///</returns>
		///
		///<param name="behavior">One of the <see cref="T:System.Data.CommandBehavior"></see> values. </param>
		/// <filterpriority>2</filterpriority>
		public override IDataReader ExecuteReader(CommandBehavior behavior)
		{
			Trap.trap();
			return new WrUnknownDataReader(cmd.ExecuteReader(behavior));
		}

		///<summary>
		///Gets or sets the text command to run against the data source.
		///</summary>
		///
		///<returns>
		///The text command to execute. The default value is an empty string ("").
		///</returns>
		///<filterpriority>2</filterpriority>
		public override string CommandText
		{
			get { return cmd.CommandText; }
			set { cmd.CommandText = value; }
		}
	}
}
