// Created by Windward Studios - no copyright is claimed. This code can be used in
// any manner by anyone for any reason. There is no copyright of any kind on it. You may
// use it in commercial products. You may change it without sharing those changes.
// We ask that you keep the "created by Windward Studios" in a comment at the top.

using System;
using System.Data;
using Kailua.net.windward.utils;

namespace net.windward.utils.ado.Unknown
{
	/// <summary>
	/// Wrapper for IDataReader. This should work for most vendors.
	/// </summary>
	class WrUnknownDataReader : WrDataReader
	{
		/// <summary>
		/// Create a IDataReader wrapper object.
		/// </summary>
		/// <param name="data">The IDataReader object to wrap.</param>
		public WrUnknownDataReader(IDataReader data) : base(data)
		{
		}

		/// <summary>
		/// Read a timestamp field and return it as an Int64.
		/// </summary>
		/// <param name="ordinal">The ordinal of the timestamp column.</param>
		/// <returns>The timestamp with [0] as msw and [1] as lsw.</returns>
		public override long GetTimestamp(int ordinal)
		{
			Trap.trap();
			if (reader.GetFieldType(ordinal) == typeof(DateTime))
			{
				DateTime dt = reader.GetDateTime(ordinal);
				return dt.Ticks;
			}
			byte[] data = new byte[8];
			reader.GetBytes(ordinal, 0, data, 0, 8);

			Int64 rtn = 0;
			for (int ind = 0; ind < 8; ind++)
			{
				rtn <<= 8;
				rtn |= data[ind];
			}

			return rtn;
		}
	}
}
