// Created by Windward Studios - no copyright is claimed. This code can be used in
// any manner by anyone for any reason. There is no copyright of any kind on it. You may
// use it in commercial products. You may change it without sharing those changes.
// We ask that you keep the "created by Windward Studios" in a comment at the top.

using System.Data.Common;

namespace net.windward.utils.ado.Unknown
{
	/// <summary>
	/// An instance of an unknown vendor's database running on a computer.
	/// </summary>
	public class WrUnknownServer : IWrServer
	{
		private readonly string name;
		private readonly DbProviderFactory providerFactory;

		/// <summary>
		/// Create an unknown DB server object.
		/// </summary>
		/// <param name="providerFactory">The .NET factory for this vendor</param>
		/// <param name="name">The name of the server.</param>
		public WrUnknownServer(DbProviderFactory providerFactory, string name)
		{
			this.name = name;
			this.providerFactory = providerFactory;
		}

		/// <summary>
		/// The name of the server.
		/// </summary>
		public string Name
		{
			get { return name; }
		}

		/// <summary>
		/// All databases on this server.
		/// </summary>
		/// <param name="credentials">The user credentials to access the database.</param>
		/// <returns>All databases on this server.</returns>
		public IWrDatabase[] GetDatabases(WrCredentials credentials)
		{
			return new WrUnknownDatabase[0];
		}

		/// <summary>
		/// All databases on this server.
		/// </summary>
		/// <param name="connStr">The connection string to access the database.</param>
		/// <returns>All databases on this server.</returns>
		public IWrDatabase[] GetDatabases(string connStr)
		{
			return new WrUnknownDatabase[0];
		}

		#region IWrServer Members

		/// <summary>
		/// Create a database object.
		/// </summary>
		/// <param name="database">The name of the database.</param>
		public IWrDatabase CreateDatabase(string database)
		{
			return new WrUnknownDatabase(providerFactory, name, database);
		}

		/// <summary>
		/// Compares the current instance with another object of the same type.
		/// </summary>
		/// <param name="obj">An object to compare with this instance. </param>
		/// <returns>a 32-bit signed integer that indicates the relative order of the objects being compared.</returns>
		public int CompareTo(object obj)
		{
			WrUnknownServer db = obj as WrUnknownServer;
			if (db == null)
				return 1;
			return name.CompareTo(db.name);
		}
		#endregion
	}
}
